import 'dart:io';

import 'package:jt_flutter_cli/create_model/utils/helpers.dart';

import 'package:path/path.dart' as path;
import 'package:yaml/yaml.dart';

void grantPermissions(String folderPath) {
  Process.runSync('chmod', ['-R', 'u+rX', folderPath]);
}

String toPascalCase(String input) {
  List<String> words = input.split(RegExp(r'[^a-zA-Z0-9]'));
  words.removeWhere((element) => element.isEmpty);
  return words
      .map((word) => word[0].toUpperCase() + word.substring(1).toLowerCase())
      .join('');
}

String capitalizeFirstCharacter(String input) {
  if (input.isEmpty) {
    return '';
  }
  String camelCaseInput = camelCase(input);

  return camelCaseInput[0].toLowerCase() + camelCaseInput.substring(1);
}

bool doesFileExist(String filePath) {
  File file = File(filePath);
  return file.existsSync();
}

void renameFilesWithSubstring(String path, String substring, String newName) {
  Directory(path).listSync(recursive: true).forEach((entity) {
    if (entity is File) {
      String fileName =
          entity.path.split(Platform.pathSeparator).last.toLowerCase();
      if (fileName.contains(substring.toLowerCase())) {
        String newPath = entity.path
            .replaceAll(substring.toLowerCase(), newName.toLowerCase());
        entity.renameSync(newPath);
      }
    }
  });
}

void processFiles(
    String path, String substring, String replacement, bool isPascalCase) {
  String pascalCase = toPascalCase(replacement);
  Directory(path).listSync(recursive: true).forEach((entity) {
    if (entity is File) {
      File file = entity;
      String content = file.readAsStringSync();
      content = content.replaceAll(
          substring, isPascalCase ? pascalCase : replacement);
      file.writeAsStringSync(content);
    }
  });
}

String? getPubspecInfo(String fieldName) {
  try {
    String currentDirectory = Directory.current.path;
    String pubspecPath = '$currentDirectory/pubspec.yaml';
    File pubspecFile = File(pubspecPath);
    String contents = pubspecFile.readAsStringSync();
    var yaml = loadYaml(contents);
    return yaml[fieldName];
  } catch (e) {
    print('Error reading pubspec.yaml: $e');
    return null;
  }
}

String getPackageVersion(String packageName) {
  String packageDirPath = Directory.current.path;
  File pubspecFile = File('$packageDirPath/pubspec.yaml');
  if (!pubspecFile.existsSync()) {
    print('pubspec.yaml file not found.');
    return '';
  }
  String pubspecContent = pubspecFile.readAsStringSync();
  var yaml = loadYaml(pubspecContent);
  var dependencies = yaml['dependencies'];
  if (dependencies != null && dependencies[packageName] != null) {
    var version = dependencies[packageName];
    String versionString = version.toString();
    if (versionString.startsWith('^')) {
      versionString = versionString.substring(1);
    }
    return versionString;
  } else {
    print('$packageName is not a dependency in pubspec.yaml.');
    return '';
  }
}

Future<void> addImport(String filePath, List<String> importStatements) async {
  final file = File(filePath);
  final lines = await file.readAsLines();

  for (var importStatement in importStatements) {
    if (!lines.contains(importStatement)) {
      lines.insert(0, importStatement);
    }
  }
  await file.writeAsString(lines.join('\n'));
}

Future<void> addCodeAboveBraces(
    String filePath, List<String> linesToAdd) async {
  final file = File(filePath);
  List<String> lines = await file.readAsLines();

  int closingBraceIndex = lines.lastIndexOf('}');
  if (closingBraceIndex == -1) {
    print('Closing brace not found.');
    return;
  }
  lines.insertAll(closingBraceIndex, linesToAdd);
  await file.writeAsString(lines.join('\n'));
}

Future<void> findAndReplace(
    String filePath, String searchString, String replacement) async {
  try {
    final file = File(filePath);
    List<String> lines = await file.readAsLines();
    for (int i = 0; i < lines.length; i++) {
      if (lines[i].contains(searchString)) {
        lines[i] = lines[i].replaceAll(searchString, replacement);
      }
    }
    await file.writeAsString(lines.join('\n'));
  } catch (e) {
    print('Error: $e');
  }
}

Future<void> appendLinesToFile(
    String filePath, List<String> linesToAppend) async {
  final file = File(filePath).openWrite(mode: FileMode.append);
  for (String line in linesToAppend) {
    file.write('$line\n');
  }
  await file.close();
}

void createFolder(String path) {
  Directory newFolder = Directory(path);

  if (!newFolder.existsSync()) {
    newFolder.createSync(recursive: true);
  }
}

Future<void> copyFolder(String sourcePath, String destinationPath) async {
  Directory sourceDir = Directory(sourcePath);
  Directory destDir = Directory(destinationPath);
  if (!await destDir.exists()) {
    await destDir.create(recursive: true);
  }
  List<FileSystemEntity> contents = sourceDir.listSync();
  for (var entity in contents) {
    String newPath = path.join(destinationPath, path.basename(entity.path));
    if (entity is Directory) {
      await copyFolder(entity.path, newPath);
    } else if (entity is File) {
      await entity.copy(newPath);
    }
  }
}
